import React, { useMemo } from "react";
import { getQualifierObject } from "@gooddata/react-components/dist/helpers/model/utils";
import { MEASURE, ATTRIBUTE } from "../utils/bucketItems";

const getTypeByIdentifier = (identifier, { attributes = [], measures = [] }) => {
    const measureMatch = measures.find(measure => measure.definition.measure.item.identifier === identifier);
    if (measureMatch) {
        return MEASURE;
    }
    const attributeMatch = attributes.find(attribute => attribute.displayForm.identifier === identifier);
    if (attributeMatch) {
        return ATTRIBUTE;
    }
    return undefined;
};

const normalizeDrillPoint = ({ title, header: { identifier, uri }, id }, executionContext) => {
    const type = getTypeByIdentifier(identifier, executionContext);
    const normalizedPoint = {
        title,
        identifier,
        uri,
        type,
    };
    if (type === ATTRIBUTE) {
        normalizedPoint.elementUri = `${uri}/elements?id=${id}`;
    }
    return normalizedPoint;
};

const normalizeDrillContext = ({ drillContext, executionContext }) => {
    if (drillContext.intersection) {
        const { element, type, y: value, intersection } = drillContext;
        return {
            element,
            type,
            value,
            intersection: intersection.map(point => {
                return normalizeDrillPoint(point, executionContext);
            }),
        };
    }
    if (drillContext.points) {
        const { element, type, points } = drillContext;
        const [value, intersection] = points.reduce(
            ([value, intersection], point) => {
                const newIntersection = point.intersection.reduce((newIntersection, intersectionPoint) => {
                    const normalizedPoint = normalizeDrillPoint(intersectionPoint, executionContext);
                    const isUnique = newIntersection.every(
                        ip =>
                            ip.identifier !== normalizedPoint.identifier ||
                            normalizedPoint.elementUri !== ip.elementUri,
                    );
                    if (isUnique) {
                        intersection.push(normalizedPoint);
                    }
                    return intersection;
                }, intersection);
                return [value + point.y, newIntersection];
            },
            [0, []],
        );
        return {
            element,
            type,
            value,
            intersection,
        };
    }
    return null;
};

const defaultChildrenFunction = ({ onFiredDrillEvent, drillableItems }) => {
    console.log({ onFiredDrillEvent, drillableItems });
    return (
        <div>
            Use children function, e.g. <code>{`(drillProps) => <ColumnChart ... {...drillProps} />`}</code>
        </div>
    );
};

const Drill = ({ on, children = defaultChildrenFunction }) => {
    const childrenProps = useMemo(() => {
        return {
            onFiredDrillEvent: drillParams => {
                const drillContext = normalizeDrillContext(drillParams);
                if (!drillContext) {
                    return;
                }
                const matchingQualifierStrings = Object.keys(on).filter(qualifierString => {
                    return drillContext.intersection.some(intersectionPoint => {
                        return (
                            intersectionPoint.identifier === qualifierString ||
                            intersectionPoint.uri === qualifierString ||
                            intersectionPoint.elementUri === qualifierString
                        );
                    });
                });
                matchingQualifierStrings.forEach(qualifierString =>
                    on[qualifierString](drillContext, drillParams.executionContext),
                );
            },
            drillableItems: Object.keys(on).map(qualifierString => {
                return getQualifierObject(qualifierString);
            }),
        };
    }, [on]);

    return children(childrenProps);
};

export default Drill;
