import React, { useRef } from "react";
import Downshift from "downshift";
import cx from "classnames";

import styles from "./Select.module.scss";
import IconChevronUp from "../icons/IconChevronUp";
import IconChevronDown from "../icons/IconChevronDown";

const disableEscapeKey = callback => event => {
    if (event.key === "Escape") {
        event.preventDefault();
    } else {
        return callback(event);
    }
};

export const ToggleButton = ({
    className = null,
    placeholder,
    downshiftProps,
    disabled = false,
    ...restProps
}) => {
    const inputRef = useRef();
    const downshiftToggleButtonProps = downshiftProps.getToggleButtonProps({ disabled });
    const toggleButtonProps = {
        ...downshiftToggleButtonProps,
        onKeyDown: disableEscapeKey(downshiftToggleButtonProps.onKeyDown),
        tabIndex: -1,
    };
    const downshiftInputProps = downshiftProps.getInputProps({
        onFocus: () => {
            downshiftProps.openMenu();
        },
        disabled,
    });
    const inputProps = {
        passRef: inputRef,
        readOnly: true,
        title: downshiftProps.inputValue,
        placeholder: placeholder,
        isOpen: downshiftProps.isOpen,
        ...downshiftInputProps,
        onKeyDown: disableEscapeKey(downshiftInputProps.onKeyDown),
    };
    return (
        <>
            <div
                className={cx(
                    className,
                    styles.Controls,
                    downshiftProps.isOpen ? styles.Open : styles.Closed,
                )}
                {...restProps}
            >
                <Input {...inputProps} />
                <label
                    {...downshiftProps.getLabelProps()}
                    className={cx(styles.Label, disabled && styles.isDisabled)}
                    onKeyDown={event => {
                        if (event.key === "Escape") {
                            event.preventDefault();
                        }
                    }}
                    onClick={e => {
                        if (downshiftProps.isOpen) {
                            downshiftProps.closeMenu();
                            e.preventDefault();
                        }
                    }}
                >
                    {downshiftProps.inputValue || placeholder}
                </label>

                {!disabled && (
                    <button {...toggleButtonProps} className={styles.IconButton}>
                        {downshiftProps.isOpen ? (
                            <IconChevronUp className={styles.Icon} />
                        ) : (
                            <IconChevronDown className={styles.Icon} />
                        )}
                    </button>
                )}
            </div>
        </>
    );
};
export const Menu = React.forwardRef(({ children, ...restProps }, ref) => (
    <div className={styles.Menu} {...restProps} ref={ref}>
        <div className={styles.MenuContent}>{children}</div>
    </div>
));
export const Input = ({ isOpen = false, passRef, ...restProps }) => (
    <input
        ref={passRef}
        className={cx(styles.Input, isOpen ? styles.Open : styles.Closed)}
        type="text"
        {...restProps}
    />
);
export const defaultItemToString = item => (item && item.label) || "";
export const Item = ({
    children,
    itemToString = defaultItemToString,
    isHighlighted = false,
    isSelected = false,
    className = null,
    ...restProps
}) => (
    <div
        className={cx(
            className,
            styles.Item,
            isHighlighted && styles.Highlighted,
            isSelected && styles.Selected,
        )}
        {...restProps}
    >
        {children}
    </div>
);

const Select = ({
    items = [],
    disabled = false,
    onChange = value => console.log("selected", value),
    initialIsOpen = false,
    itemToString = defaultItemToString,
    placeholder = "Select",
    selectedItem = null,
    className = null,
    style = {},
    itemClassName = null,
    itemStyle = {},
    ItemComponent = Item,
    ToggleButtonComponent = ToggleButton,
}) => {
    return (
        <Downshift
            selectedItem={selectedItem}
            onChange={onChange}
            defaultHighlightedIndex={0}
            initialIsOpen={initialIsOpen}
            itemToString={itemToString}
            itemCount={items.length}
        >
            {downshiftProps => {
                const { getMenuProps, getItemProps, isOpen, selectedItem, highlightedIndex } = downshiftProps;
                return (
                    <div className={cx(styles.Select, className)} style={style}>
                        <ToggleButtonComponent
                            placeholder={placeholder}
                            downshiftProps={downshiftProps}
                            disabled={disabled}
                        />
                        {isOpen && !disabled ? (
                            <Menu {...getMenuProps({ open: isOpen })}>
                                {items.map((item, index) => {
                                    return (
                                        <ItemComponent
                                            key={index}
                                            isHighlighted={highlightedIndex === index}
                                            {...getItemProps({
                                                item,
                                                index,
                                                isSelected: selectedItem === item,
                                                className: itemClassName,
                                                style: itemStyle,
                                            })}
                                        >
                                            {itemToString(item)}
                                        </ItemComponent>
                                    );
                                })}
                            </Menu>
                        ) : null}
                    </div>
                );
            }}
        </Downshift>
    );
};

export default Select;
