import React, { useCallback, useState, useEffect } from "react";
import Downshift from "downshift";
import Fuse from "fuse.js";
import cx from "classnames";

import styles from "./SelectWithSearch.module.scss";
import ChevronUp from "../icons/ChevronUp";
import ChevronDown from "../icons/ChevronDown";

export const Label = ({ children, ...restProps }) => (
    <label className={styles.Label} {...restProps}>
        {children}
    </label>
);
export const Menu = React.forwardRef(({ children, ...restProps }, ref) => (
    <div className={styles.Menu} {...restProps} ref={ref}>
        <div className={styles.MenuContent}>{children}</div>
    </div>
));
export const ToggleButton = ({ onClick, isOpen, ...restProps }) => (
    <button className={styles.ToggleButton} onClick={onClick} type="button" {...restProps}>
        {isOpen ? <ChevronUp className={styles.Icon} /> : <ChevronDown className={styles.Icon} />}
    </button>
);
export const Input = ({ value = null, isOpen = false, ...restProps }) => (
    <input className={styles.Input} type="text" value={value.toString()} {...restProps} />
);
export const defaultItemToString = item => (item && item.label) || "";
export const Item = ({
    children,
    itemToString = defaultItemToString,
    isHighlighted = false,
    isSelected = false,
    className = null,
    ...restProps
}) => (
    <div
        className={cx(
            className,
            styles.Item,
            isHighlighted && styles.Highlighted,
            isSelected && styles.Selected,
        )}
        {...restProps}
    >
        {children}
    </div>
);

export const defaultGetItems = (items = [], inputString = "") => {
    console.log("defaultGetItems", items, inputString);
    if ((inputString ? inputString.trim() : "") === "") {
        console.log("no input, return all items", items);
        return items;
    }
    const itemsFuse = new Fuse(items, {
        keys: ["label"],
    });
    const searchResult = itemsFuse.search(inputString);
    console.log("search result", searchResult);
    return searchResult;
};

const SelectWithSearch = ({
    getItems = defaultGetItems,
    items = [],
    label = null,
    onChange = value => console.log("selected", value),
    initialIsOpen = false,
    itemToString = defaultItemToString,
    placeholder = undefined,
    selectedItem = null,
    className = null,
    style = {},
    itemClassName = null,
    itemStyle = {},
    clearOnFocus = true,
    ItemComponent = Item,
    ToggleButtonComponent = ToggleButton,
}) => {
    const [lastSelectedItem, setLastSelectedItem] = useState(null);
    useEffect(() => {
        if (selectedItem !== null) {
            setLastSelectedItem(selectedItem);
        }
    }, [items, selectedItem]);
    const onChangeCallback = useCallback(selectedItem => onChange(selectedItem), [onChange]);
    return (
        <Downshift
            selectedItem={selectedItem}
            onChange={onChangeCallback}
            initialIsOpen={initialIsOpen}
            itemToString={itemToString}
        >
            {({
                getLabelProps,
                getInputProps,
                getToggleButtonProps,
                getMenuProps,
                getItemProps,
                isOpen,
                clearSelection,
                selectedItem,
                selectItem,
                inputValue,
                highlightedIndex,
                openMenu,
            }) => (
                <div className={cx(styles.Select, className)} style={style}>
                    {label && <Label {...getLabelProps()}>{label}</Label>}
                    <div className={styles.Controls}>
                        <Input
                            isOpen={isOpen}
                            {...getInputProps({
                                placeholder,
                                onFocus: () => {
                                    if (clearOnFocus) {
                                        clearSelection();
                                    }
                                    if (!isOpen) {
                                        openMenu();
                                    }
                                },
                                onBlur: event => {
                                    if (clearOnFocus && event.target.value === "" && lastSelectedItem) {
                                        selectItem(lastSelectedItem);
                                    }
                                },
                            })}
                        />
                        <ToggleButtonComponent
                            isOpen={isOpen}
                            {...getToggleButtonProps({
                                onClick: () => {
                                    if (clearOnFocus && !isOpen) {
                                        clearSelection();
                                    }
                                    // re-selecting last item here is in conflict with onBlur and behaves unpredictably depending on if the input was selected or not
                                },
                            })}
                        />
                    </div>
                    {isOpen ? (
                        <Menu {...getMenuProps({ open: isOpen })}>
                            {getItems(items, inputValue).map((item, index) => {
                                return (
                                    <ItemComponent
                                        key={index}
                                        isHighlighted={highlightedIndex === index}
                                        {...getItemProps({
                                            item,
                                            index,
                                            isSelected: selectedItem === item,
                                            className: itemClassName,
                                            style: itemStyle,
                                        })}
                                    >
                                        {itemToString(item)}
                                    </ItemComponent>
                                );
                            })}
                        </Menu>
                    ) : null}
                </div>
            )}
        </Downshift>
    );
};

export default SelectWithSearch;
