import React, { useState, useCallback, memo, useEffect } from "react";
import cx from "classnames";

import CatalogueConsumer from "../../CatalogueConsumer";
import { visTypes as allVisTypes } from "./visTypes";
import VisualizationBuilderHeader from "./VisualizationBuilderHeader";
import VisualizationBuilderBucket from "./VisualizationBuilderBucket";
import VisualizationBuilderMessages from "./VisualizationBuilderMessages";
import VisualizationBuilderVisualization from "./VisualizationBuilderVisualization";

import styles from "./VisualizationBuilderWidget.module.scss";
import sharedStyles from "../../../shared.module.scss";
import { ATTRIBUTE_AND_DATE, ATTRIBUTE, DATE } from "../../../utils/bucketItems";
import { getCatalogItemsByType } from "../../../utils/catalogue";

const getBucketErrors = (buckets, visType) => {
    if (!visType) {
        return [
            {
                bucket: null,
                message: `First select a visualization.`,
            },
        ];
    }

    const requiredBucketsEmpty = buckets
        .filter(bucket => bucket.isRequired && bucket.items.length === 0)
        .map(bucket => ({
            bucket,
            message: `Bucket ${bucket.label} is required but empty.`,
        }));

    const tooManyItems = buckets
        .filter(bucket => bucket.items.length > bucket.limit)
        .map(bucket => ({
            bucket,
            message: `Bucket ${bucket.label} has too many items. Maximum is ${bucket.limit} items.`,
        }));

    const errors = [...requiredBucketsEmpty, ...tooManyItems];

    const filledBucketCount = buckets.filter(bucket => bucket.items.length > 0).length;

    if (filledBucketCount < visType.minimumBuckets) {
        errors.push({
            bucket: null,
            message: `At least ${visType.minimumBuckets} bucket${visType.minimumBuckets > 1 ? "s" : ""} ${
                visType.minimumBuckets > 1 ? "have" : "has"
            } to be filled.`,
        });
    }

    return errors;
};

const isBucketTypeSubset = (type, requiredType) => {
    if (requiredType === ATTRIBUTE_AND_DATE) {
        return type === ATTRIBUTE || type === DATE;
    }
    return type === requiredType;
};

const matchBucket = (bucketIndex, buckets, prevBuckets) => {
    const bucket = buckets[bucketIndex];
    const bucketMatchingProp = prevBuckets.find(
        prevBucket => prevBucket.prop === bucket.prop && isBucketTypeSubset(prevBucket.type, bucket.type),
    );

    if (bucketMatchingProp) {
        return bucketMatchingProp;
    }

    // if cannot find exact match, try finding buckets of same type and type index
    const bucketsOfType = buckets.filter(prevBucket => prevBucket.type === bucket.type);
    const prevBucketsOfType = prevBuckets.filter(prevBucket =>
        isBucketTypeSubset(prevBucket.type, bucket.type),
    );
    const bucketTypeIndex = bucketsOfType.indexOf(bucket);
    const bucketTypeOrderMatch = prevBucketsOfType[bucketTypeIndex];

    if (bucketTypeOrderMatch) {
        return bucketTypeOrderMatch;
    }
    return null;
};

const assignBucketItems = (buckets, prevBuckets) => {
    return buckets.map((bucket, bucketIndex) => {
        const matchingBucket = matchBucket(bucketIndex, buckets, prevBuckets);
        const matchingItems = matchingBucket ? matchingBucket.items : [];
        const items = matchingItems.slice(0, bucket.limit);
        if (items.length > 0) {
            return {
                ...bucket,
                items,
            };
        }
        return bucket;
    });
};

const VisualizationBuilderWidget = ({
    className = null,
    visTypes = allVisTypes,
    initialVisType = allVisTypes[0],
    visualizationProps = {},
    headline = null,
    onBucketsChange = null,
    ...restProps
}) => {
    const [visType, setVisType] = useState(initialVisType);
    const [buckets, setBuckets] = useState(initialVisType.buckets);

    // reset buckets to default for a given visualization type on every visualization type change
    const switchVisualization = useCallback(
        nextVisType => {
            const newBuckets = assignBucketItems(nextVisType.buckets, buckets);
            setBuckets(newBuckets);
            setVisType(nextVisType);
        },
        [buckets],
    );

    const errors = getBucketErrors(buckets, visType);

    useEffect(() => {
        if (onBucketsChange && buckets) {
            onBucketsChange(buckets)
        }
        // Do not update on onBucketsChange
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [buckets]);

    return (
        <div className={cx(className, sharedStyles.Widget, styles.VisualizationBuilderWidget)} {...restProps}>
            <VisualizationBuilderHeader
                headline={headline}
                selectedVisType={visType}
                setVisType={switchVisualization}
                visTypes={visTypes}
            />
            <VisualizationBuilderMessages errors={errors} />
            <CatalogueConsumer onData={(data) => {
                let containsInvalidBucketItems = false;
                // @ts-ignore
                const validBuckets = buckets.map(bucket => {
                    const catalogueItems = getCatalogItemsByType(data, bucket.type);
                    const validBucketItems = bucket.items.filter(bucketItem => {
                        return !!catalogueItems.find(catalogueItem =>
                            !!bucket.matchCatalogItem(bucketItem, catalogueItem),
                        );
                    })
                    if (bucket.items.length !== validBucketItems.length) {
                        containsInvalidBucketItems = true;
                        return {
                            ...bucket,
                            items: validBucketItems
                        };
                    }
                    return bucket;
                });
                if (containsInvalidBucketItems) {
                    setBuckets(validBuckets);
                }
            }}>
                {catalogue => {
                    const { projectId } = catalogue;
                    return (
                        <>
                            {buckets.length > 0 && (
                                <div className={styles.Buckets}>
                                    {buckets.map((bucket, bucketIndex) => (
                                        <VisualizationBuilderBucket
                                            bucket={bucket}
                                            bucketIndex={bucketIndex}
                                            buckets={buckets}
                                            catalogue={catalogue}
                                            errors={errors}
                                            setBuckets={setBuckets}
                                            key={bucket.prop}
                                        />
                                    ))}
                                </div>
                            )}
                            <VisualizationBuilderVisualization
                                visType={visType}
                                buckets={buckets}
                                className={styles.VisualizationWrapper}
                                projectId={projectId}
                                visualizationProps={visualizationProps}
                                errors={errors}
                            />
                        </>
                    );
                }}
            </CatalogueConsumer>
        </div>
    );
};

export default memo(VisualizationBuilderWidget);
